#define MOZ_SOURCE_STAMP 829fcc7f60f04dffff4d29b007ef8015a1cd2c99
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/829fcc7f60f04dffff4d29b007ef8015a1cd2c99
